@@ -28,6 +28,7 @@ from intermine.model import Model, Attribute, Reference, Collection, Column
 from intermine.lists.listmanager import ListManager
 from intermine.errors import ServiceError, WebserviceError
 from intermine.results import InterMineURLOpener, ResultIterator
+from intermine import idresolution
 
 """
 Webservice Interaction Routines for InterMine Webservices
@@ -48,8 +49,8 @@ class Registry(object, UserDict.DictMixin):
     A Class representing an InterMine registry.
     ===========================================
 
-    Registries are web-services that mines can automatically register themselves
-    with, and thus enable service discovery by clients.
+    Registries are web-services that mines can automatically register
+    themselves with, and thus enable service discovery by clients.
 
     SYNOPSIS
     --------
@@ -194,6 +195,7 @@ class Service(object):
     RELEASE_PATH           = '/version/release'
     SCHEME                 = 'http://'
     SERVICE_RESOLUTION_PATH = "/check/"
+    IDS_PATH               = "/ids"
 
     def __init__(self, root,
             username=None, password=None, token=None,
@@ -449,6 +451,54 @@ class Service(object):
            self._widgets = dict(([w['name'], w] for w in data['widgets']))
         return self._widgets
 
+    def resolve_ids(self, data_type, identifiers, extra = '', case_sensitive = False, wildcards = False):
+        """
+        Submit an Identifier Resolution Job
+        ===================================
+
+        Request that a set of identifiers be resolved to objects in
+        the data store.
+
+        @param data_type: The type of these identifiers (eg. 'Gene')
+        @type data_type: String
+
+        @param identifiers: The ids to resolve (eg. ['eve', 'zen', 'pparg'])
+        @type identifiers: iterable of string
+
+        @param extra: A disambiguating value (eg. "Drosophila melanogaster")
+        @type extra: String
+
+        @param case_sensitive: Whether to treat IDs case sensitively.
+        @type case_sensitive: Boolean
+
+        @param wildcards: Whether or not to interpret wildcards (eg: "eve*")
+        @type wildcards: Boolean
+
+        @return: {idresolution.Job} The job.
+        """
+        if self.version < 10:
+            raise ServiceError("This feature requires API version 10+")
+        if not data_type:
+            raise ServiceError("No data-type supplied")
+        if not identifiers:
+            raise ServiceError("No identifiers supplied")
+
+        data = json.dumps({
+            "type": data_type,
+            "identifiers": list(identifiers),
+            "extra": extra,
+            "caseSensitive": case_sensitive,
+            "wildCards": wildcards
+        })
+        text = self.opener.post_content(self.root + self.IDS_PATH, data, InterMineURLOpener.JSON)
+        ret = json.loads(text)
+        if ret['error'] is not None:
+            raise ServiceError(ret['error'])
+        if ret['uid'] is None:
+            raise Exception("No uid found in " + ret)
+
+        return idresolution.Job(self, ret['uid'])
+
     def flush(self):
         """
         Flushes any cached data.
